# Alpha houses all of our containerized applications
# Essentially it is a cluster with services that we
# choose to expose to the internet in one form or another

# Logging Configuration for services inside the cluster
#######################################################

locals {
  subnet = "subnet-09302319a6678643f"
}
# Alpha Cloudwatch logging configuration
########################################
resource "aws_cloudwatch_log_group" "alpha" {
  name = "${var.athens_prefix}-alpha-log"
  retention_in_days = 7
}

# Alpha logging role 
#####################
resource "aws_iam_role" "alpha_iam_role" {
  name = "${var.athens_prefix}-alpha-iam-role"
  assume_role_policy = jsonencode({
    Version = "2012-10-17"
    Statement = [
      {
        Action = "sts:AssumeRole"
        Principal = {
          Service = [ "ecs-tasks.amazonaws.com" ]
        }
        Effect = "Allow"
      }
    ]
  })
}

resource "aws_iam_policy" "alpha_iam_policy" {
  name = "${var.athens_prefix}-alpha-iam-policy"
  policy = jsonencode({
    Version = "2012-10-17"
    Statement = [
      {
        Effect = "Allow"
        Action = [
          "logs:CreateLogGroup",
          "logs:CreateLogStream",
          "logs:DescribeLogStreams",
          "logs:PutLogEvents",
        ]
        "Resource" = "*"
      }
    ]
  })
}

resource "aws_iam_role_policy_attachment" "alpha_logs" {
  role        = aws_iam_role.alpha_iam_role.name
  policy_arn  = aws_iam_policy.alpha_iam_policy.arn
}


# Alpha cluster definition 
###########################
resource "aws_ecs_cluster" "alpha" {
  name = "${var.athens_prefix}-athens-alpha"
  configuration {
    execute_command_configuration {
      logging = "OVERRIDE"
      log_configuration {
        cloud_watch_log_group_name = aws_cloudwatch_log_group.alpha.name
      }
    }
  }
}


# Lewdlad Service Definition
############################
resource "aws_ecs_task_definition" "sample" {
  family = "${var.athens_prefix}-sample"

  network_mode = "awsvpc"
  requires_compatibilities = ["FARGATE"]
  execution_role_arn = aws_iam_role.alpha_iam_role.arn

  cpu = 256
  memory = 512

  container_definitions = jsonencode([
    {
      name = "${var.athens_prefix}-sample-container"
      image = "nginxdemos/nginx-hello:latest"
      cpu = 128
      memory = 256
      essential = true
      portMappings = [
        {
          containerPort = 8080
          hostPort = 8080
        }
      ]
      logConfiguration = {
        logDriver = "awslogs"
        options = {
          awslogs-group  = aws_cloudwatch_log_group.alpha.name
          awslogs-region = "us-west-1"
          awslogs-stream-prefix  = "sample-container"
        }
      }
    }
  ])
  tags = {
    Name = "${var.athens_prefix}-sample-task-def-container"
  }
}

# Service level definition
##########################
resource "aws_ecs_service" "sample" {
  name            = "${var.athens_prefix}-sample-service"
  cluster         = aws_ecs_cluster.alpha.id
  task_definition = aws_ecs_task_definition.sample.arn
  desired_count   = 1
  launch_type = "FARGATE"
  network_configuration {
    assign_public_ip = true
    subnets          = [ local.subnet ]
    security_groups  = [ 
      aws_security_group.ecs_web_ingress.id,
      aws_security_group.base_ecs.id
      ]
  }
}
